""" Formats repositories in console """

# Standard library imports
import textwrap

# Third party imports
from colorama import init, Fore, Style
from tabulate import tabulate


def make_hyperlink(text, target):
    """ Makes hyperlink out of text and target and retuns it
        https://stackoverflow.com/questions/44078888/clickable-html-links-in-python-3-6-shell
    """
    return f"\u001b]8;;{target}\u001b\\{text}\u001b]8;;\u001b\\"


def colored_output(repos):
    """ Displays repositories using colorama """

    init()  # initialize coloroma
    seperator = "+==================================================================+"
    print(Fore.WHITE, Style.BRIGHT, seperator, end="\n\n")
    for repo in repos:
        print(
            Fore.LIGHTRED_EX,
            Style.BRIGHT,
            f"{make_hyperlink(repo['name'], repo['html_url'])}",
        )
        print(
            Fore.LIGHTYELLOW_EX,
            Style.NORMAL,
            "\n  ".join(textwrap.wrap(f"{repo['description']}", len(seperator))),
            end="\n\n",
        )
        print(Fore.LIGHTCYAN_EX, Style.BRIGHT, f"{repo['language']}", end="\t")
        print(
            Fore.LIGHTCYAN_EX,
            Style.BRIGHT,
            f"{repo['stargazers_count']} Stars",
            end="\t",
        )
        print(Fore.LIGHTCYAN_EX, Style.BRIGHT, f"{repo['forks_count']} Forks", end="\t")
        print(
            Fore.LIGHTCYAN_EX,
            Style.BRIGHT,
            f"{repo['watchers_count']} Watchers",
            end="\n\n",
        )
        print(Fore.WHITE, Style.BRIGHT, seperator, end="\n\n")


def tabular_output(repos):
    """ Displays repositories as tables using tabulate """
    table_headers = ["URL", "Language", "Stars", "Forks", "Watches"]
    repositories = [
        [
            repo["html_url"],
            repo["language"],
            repo["stargazers_count"],
            repo["forks_count"],
            repo["watchers_count"],
        ]
        for repo in repos
    ]
    print(tabulate(repositories, headers=table_headers, tablefmt="fancy_grid"))


def beautify(repos, fmt):
    """ Beautfies the output based on display format given """
    if fmt == "colored":
        colored_output(repos)
    elif fmt == "table":
        tabular_output(repos)
    else:
        print("Can't output anything. Invalid display format!")



""" githunt.__main__ """

# Standard library imports
from datetime import datetime, timedelta

# Third party imports
import click
import requests

# Internal application imports
from .utils import beautify

API_URL = "https://api.github.com/search/repositories"


@click.command()
@click.option("--language", "-l", default="", help="language filter (eg: python)")
@click.option(
    "--date",
    "-d",
    default="",
    help="date in the ISO8601 format which is YYYY-MM-DD (year-month-day)",
)
@click.option(
    "--fmt",
    "-f",
    default="colored",
    help="output format, it can be either table or colored",
)
def search(language, date, fmt):
    """ Returns repositories based on the language.
        repositories are sorted by stars
    """

    if not date:
        start_date = datetime.fromisoformat(
            datetime.utcnow().date().isoformat()
        )  # today's timestamp in YYYY-MM-DD:00:00:00 format
        end_date = datetime.fromisoformat(
            (datetime.utcnow() + timedelta(days=1)).date().isoformat()
        )  # next day's timestamp in YYYY-MM-DD:00:00:00 format
    else:
        start_date = datetime.fromisoformat(date)
        end_date = datetime.fromisoformat(
            (start_date + timedelta(days=1)).date().isoformat()
        )

    query = f"stars:>0+created:{start_date.isoformat()}..{end_date.isoformat()}"
    query += f"+language:{language}" if language else ""
    url = f"{API_URL}?q={query}&sort=stars&order=desc"
    repositories = requests.get(url).json()
    beautify(repositories["items"], fmt)


if __name__ == "__main__":
    # pylint: disable=no-value-for-parameter
    search()



import io
import pathlib
import re

from setuptools import setup

with io.open("README.md", "rt", encoding="utf8") as f:
    readme = f.read()

with io.open("githunt/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

# This call to setup() does all the work
setup(
    name="githunt",
    version=version,
    description="Browse most stared Github repositories by date from your command line.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/SriNandan33/githunt",
    author="Srinivasa Rao",
    author_email="nandusrinivas33.cse@gmail.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["githunt"],
    include_package_data=True,
    install_requires=[
        "black==19.10b0",
        "Click==7.0",
        "colorama==0.4.3",
        "pylint==2.4.4",
        "requests==2.22.0",
        "tabulate==0.8.6",
    ],
    entry_points={
        "console_scripts": [
            "githunt=githunt.__main__:search",
        ]
    },
)